/** 
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.codehaus.activecluster;

import javax.jms.JMSException;
import javax.jms.Topic;


/**
 * A Factory of Cluster instances
 * 
 * @version $Revision: 1.3 $
 */
public interface ClusterFactory {

    /**
     * Creates a new cluster connection using the given group
     *
     * @param groupDestination
     * @return Cluster
     * @throws ClusterException
     * @throws JMSException
     */
    public Cluster createCluster(Topic groupDestination) throws ClusterException, JMSException;

    /**
     * Creates a new cluster connection using the given topic name
     *
     * @param topic is the name of the topic to use
     * @return Cluster
     * @throws ClusterException
     * @throws JMSException
     */
    public Cluster createCluster(String topic) throws ClusterException, JMSException;


}
