/** 
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.codehaus.activecluster.election.impl;

import org.codehaus.activecluster.Cluster;
import org.codehaus.activecluster.Node;
import org.codehaus.activecluster.election.ElectionStrategy;

import javax.jms.JMSException;
import java.util.Iterator;
import java.util.Map;

/**
 * <p><code>BullyElectionStrategy</code> Use a simple bully algorithm to elect a coordinator.
 * the member with the lowest lexicographical name is choosen</p>
 *
 * @version $Revision: 1.2 $
 */
public class BullyElectionStrategy implements ElectionStrategy {

    /**
     * Elect a coordinator.
     *
     * @param cluster
     * @return the elected Node
     * @throws JMSException
     */
    public Node doElection(Cluster cluster) throws JMSException {
        Node elect = cluster.getLocalNode();

        Map nodes = cluster.getNodes();
        for (Iterator i = nodes.values().iterator(); i.hasNext();) {
            Node node = (Node) i.next();
            if (elect.getName().compareTo(node.getName()) < 0) {
                elect = node;
            }
        }

        return elect;
    }

}
