/** 
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.activecluster.impl;

import org.codehaus.activecluster.Node;

import javax.jms.Destination;
import java.util.HashMap;
import java.util.Map;


/**
 * Default implementation of a remote Node
 *
 * @version $Revision: 1.6 $
 */
public class NodeImpl implements Node {

    private Destination destination;
    protected Map state;
    protected boolean coordinator;

    /**
     * Allow a node to be copied for sending it as a message
     *
     * @param node
     */
    public NodeImpl(Node node) {
        this(node.getDestination(), node.getState());
    }

    public NodeImpl(Destination destination) {
        this(destination, new HashMap());
    }

    public NodeImpl(Destination destination, Map state) {
        this.destination = destination;
        this.state = state;
    }

    /**
     * @return the name of the node
     */
    public String getName() {
        return destination.toString();
    }

    public String toString() {
        return "Node[destination: " + destination + " state: " + state + "]";
    }

    public Destination getDestination() {
        return destination;
    }

    public synchronized Map getState() {
        return new HashMap(state);
    }


    protected synchronized void setState(Map state) {
        this.state = state;
    }


    /**
     * @return true if this node has been elected as coordinator
     */
    public boolean isCoordinator() {
        return coordinator;
    }

    protected void setCoordinator(boolean value) {
        coordinator = value;
    }
}
